@@ -1,15 +1,15 @@
-# made by billythegoat356 and loTus01, with the help of BlueRed
+# made by billythegoat356, loTus01, and BlueRed
 
 # https://github.com/billythegoat356 https://github.com/loTus04 https://github.com/CSM-BlueRed
 
-# Version : 1.7
+# Version : 1.8 (added Colorate.Format, added Banner.Arrow, added Anime.Move, added Center.GroupAlign, added Center.TextAlign, updated Box to Banner, updated Box.Lines))
 
 # based on pyfade anc pycenter, R.I.P
 
 # <3
 
 
-from os import name as _name, system as _system, get_terminal_size as _terminal_size
+from os import name as _name, system as _system, get_terminal_size as _terminal_size, terminal_size
 from sys import stdout as _stdout
 from time import sleep as _sleep
 from threading import Thread as _thread
@@ -28,7 +28,6 @@ class System:
     """
     1 variable:
         Windows      |      tells if the user is on Windows OS or not
-
     5 functions:
         Init()       |      initialize the terminal to allow the use of colors
         Clear()      |      clear the terminal
@@ -64,7 +63,6 @@ class Cursor:
     2 functions:
         HideCursor()      |      hides the white blinking in the terminal
         ShowCursor()      |      shows the white blinking in the terminal
-
     """
 
     def HideCursor():
@@ -102,7 +100,7 @@ class _MakeColors:
         return col.replace('\033[38;2;', '').replace('m','').replace('50m', '').replace('\x1b[38', '')
 
     def _makergbcol(var1: list, var2: list) -> list:
-        col = [_col for _col in var1[:12]]
+        col = list(var1[:12])
         for _col in var2[:12]:
             col.append(_col)
         for _col in reversed(col):
@@ -131,7 +129,7 @@ class _MakeColors:
         return rainbow
     
     def _reverse(colors: list) -> list:
-        _colors = [col for col in colors]
+        _colors = list(colors)
         for col in reversed(_colors):
             colors.append(col)
         return colors
@@ -164,7 +162,6 @@ class Colors:
         StaticMIX()        |      mix two or more static colors
         DynamicMIX()       |      mix two or more dynamic colors
         Symbol()           |      create a colored symbol, ex: '[!]'
-
     """
 
     def StaticRGB(r: int, g: int, b: int) -> str:
@@ -194,9 +191,8 @@ class Colors:
             if colors.index(color) == len(colors) - 1:
                 break
             _colors.append([color, colors[colors.index(color) + 1]])
-        colors = []
-        for color in _colors:
-            colors.append(_MakeColors._mixcolors(col1=color[0], col2=color[1], _reverse=False))
+        colors = [_MakeColors._mixcolors(col1=color[0], col2=color[1], _reverse=False) for color in _colors]
+
         final = []
         for col in colors:
             for col in col:
@@ -292,6 +288,8 @@ class Colors:
 
     """ ! developper area ! """
 
+    col = (list, str)
+
     dynamic_colors = [
         black_to_white, black_to_red, black_to_green, black_to_blue,
         white_to_black, white_to_red, white_to_green, white_to_blue,
@@ -306,7 +304,7 @@ class Colors:
     ]
 
     for color in dynamic_colors:
-        col = 20
+        _col = 20
         reversed_col = 220
 
         dbl_col = 20
@@ -318,14 +316,14 @@ class Colors:
         for _ in range(12):
 
             if 'm' in content:
-                result = content.replace('m', str(col))
+                result = content.replace('m', str(_col))
                 color.append(result)
 
             elif 'n' in content:
                 result = content.replace('n', str(reversed_col))
                 color.append(result)
 
-            col += 20
+            _col += 20
             reversed_col -= 20
 
         for _ in range(12):
@@ -353,12 +351,11 @@ class Colors:
     rainbow = _MakeColors._makerainbow(
         red_to_green, green_to_blue, blue_to_red)
 
-    for col in [
-            red_to_blue, red_to_green,
-            green_to_blue, green_to_red,
-            blue_to_red, blue_to_green
-    ]:
-        dynamic_colors.append(col)
+    for _col in (
+        red_to_blue, red_to_green,
+        green_to_blue, green_to_red,
+        blue_to_red, blue_to_green
+    ): dynamic_colors.append(_col)
 
     dynamic_colors.append(rainbow)
 
@@ -388,16 +385,15 @@ class Colorate:
         Static colors:
             Color()                 |            color a text with a static color
             Error()                 |            make an error with red text and advanced arguments
-
+            Format()                |            set different colors for different parts of a text
         Dynamic colors:
             Vertical()              |           fade a text vertically
             Horizontal()            |           fade a text horizontally
             Diagonal()              |           fade a text diagonally
             DiagonalBackwards()     |           fade a text diagonally but backwards
-
     """
 
-    """ fix/static colors"""
+    """ fix/static colors """
 
     def Color(color: str, text: str, end: bool = True) -> str:
         return _MakeColors._maketext(color=color, text=text, end=end)
@@ -420,8 +416,7 @@ class Colorate:
 
     """ faded/dynamic colors"""
 
-    def Vertical(color: list, text: str, speed: int = 1, start: int = 0, stop: int = 0, cut: int = 0) -> str:
-
+    def Vertical(color: list, text: str, speed: int = 1, start: int = 0, stop: int = 0, cut: int = 0, fill: bool = False) -> str:
         color = color[cut:]
         lines = text.splitlines()
         result = ""
@@ -430,9 +425,14 @@ class Colorate:
         color_n = 0
         for lin in lines:
             colorR = color[color_n]
-            result += " " * \
-                _MakeColors._getspaces(
-                    lin) + _MakeColors._makeansi(colorR, lin.strip()) + "\n"
+            if fill:
+                result += " " * \
+                    _MakeColors._getspaces(
+                        lin) + "".join(_MakeColors._makeansi(colorR, x) for x in lin.strip()) + "\n"
+            else:
+                result += " " * \
+                    _MakeColors._getspaces(
+                        lin) + _MakeColors._makeansi(colorR, lin.strip()) + "\n"  
 
             if nstart != start:
                 nstart += 1
@@ -454,7 +454,6 @@ class Colorate:
         return result.rstrip()
 
     def Horizontal(color: list, text: str, speed: int = 1, cut: int = 0) -> str:
-
         color = color[cut:]
         lines = text.splitlines()
         result = ""
@@ -496,7 +495,6 @@ class Colorate:
         return result.rstrip()
 
     def DiagonalBackwards(color: list, text: str, speed: int = 1, cut: int = 0) -> str:
-
         color = color[cut:]
 
         lines = text.splitlines()
@@ -519,19 +517,30 @@ class Colorate:
             result = result + '\n' + resultL
         return result.strip()
 
+    def Format(text: str, second_chars: list, mode, principal_col: Colors.col, second_col: str):
+        if mode == Colorate.Vertical:
+            ctext = mode(principal_col, text, fill=True)
+        else:
+            ctext = mode(principal_col, text)
+        ntext = ""
+        for x in ctext:
+            if x in second_chars:
+                x = Colorate.Color(second_col, x)
+            ntext += x
+        return ntext
+
 
 class Anime:
 
     """
     2 functions:
         Fade()                  |            make a small animation with a changing color text, using a dynamic color
-        Move()                  |            make a small animation moving the text, available soon
+        Move()                  |            make a small animation moving the text from left to right
         Bar()                   |            a fully customizable charging bar
         Anime()                 |            a mix between Fade() and Move(), available soon
-
     """
 
-    def Fade(text: str, color: list, mode, time=True, interval=0.05, hide_cursor: bool = True, enter: bool = False):
+    def Fade(text: str, color: list, mode, time = True, interval = 0.05, hide_cursor: bool = True, enter: bool = False):
         if hide_cursor:
             Cursor.HideCursor()
 
@@ -542,13 +551,11 @@ class Anime:
         passed = False
 
         if enter:
-            th = _thread(target=Anime._input)
+            th = _thread(target = Anime._input)
             th.start()
 
         if time is True:
-            while True:
-                if passed is not False:
-                    break
+            while not passed:
                 Anime._anime(text, color, mode, interval)
                 ncolor = color[1:]
                 ncolor.append(color[0])
@@ -556,7 +563,7 @@ class Anime:
 
         else:
             for _ in range(time):
-                if passed is not False:
+                if passed:
                     break
                 Anime._anime(text, color, mode, interval)
                 ncolor = color[1:]
@@ -566,18 +573,61 @@ class Anime:
         if hide_cursor:
             Cursor.ShowCursor()
 
-    def Move() -> None: ...
+    def Move(text: str, color: list, time = True, interval = 0.01, hide_cursor: bool = True, enter: bool = False):
+        if hide_cursor:
+            Cursor.HideCursor()
+
+        if type(time) == int:
+            time *= 15
+
+        global passed
+        passed = False
+
+        columns = _terminal_size().columns
+
+        if enter:
+            th = _thread(target = Anime._input)
+            th.start()
+
+        count = 0
+        mode = 1
+
+        if time is True:
+            while not passed:
+                if mode == 1:
+                    if count >= (columns - (max(len(txt) for txt in text.splitlines()) + 1)):
+                        mode = 2
+                    count += 1
+                elif mode == 2:
+                    if count <= 0:
+                        mode = 1
+                    count -= 1
+                Anime._anime('\n'.join((' ' * count) + line for line in text.splitlines()), color or [], lambda a, b: b, interval)
+        else:
+            for _ in range(time):
+                if passed:
+                    break
+                if mode == 1:
+                    if count >= (columns - (max(len(txt) for txt in text.splitlines()) + 1)):
+                        mode = 2
+                elif mode == 2:
+                    if count <= 0:
+                        mode = 1
+                Anime._anime('\n'.join((' ' * count) + line for line in text.splitlines()), color or [], lambda a, b: b, interval)
+
+                count += 1
+
+        if hide_cursor:
+            Cursor.ShowCursor()
+
 
     def Bar(length, carac_0: str = '[ ]', carac_1: str = '[0]', color: list = Colors.white, mode=Colorate.Horizontal, interval: int = 0.5, hide_cursor: bool = True, enter: bool = False, center: bool = False):
-
         if hide_cursor:
             Cursor.HideCursor()
 
         if type(color) == list:
-            while True:
-                if length <= len(color):
-                    break
-                ncolor = [col for col in color]
+            while not length <= len(color):
+                ncolor = list(color)
                 for col in ncolor:
                     color.append(col)
 
@@ -590,27 +640,25 @@ class Anime:
 
         for i in range(length + 1):
             bar = carac_1 * i + carac_0 * (length - i)
-            if passed is not False:
+            if passed:
                 break
             if type(color) == list:
                 if center:
                     print(Center.XCenter(mode(color, bar)))
                 else:
                     print(mode(color, bar))
-                _sleep(interval)
-                System.Clear()
             else:
                 if center:
                     print(Center.XCenter(color + bar))
                 else:
                     print(color + bar)
-                _sleep(interval)
-                System.Clear()
-
+            _sleep(interval)
+            System.Clear()
         if hide_cursor:
             Cursor.ShowCursor()
 
     def Anime() -> None: ...
+
     """ ! developper area ! """
 
     def _anime(text: str, color: list, mode, interval: int):
@@ -630,11 +678,9 @@ class Write:
     2 functions:
         Print()         |          print a text to the terminal while coloring it and with a fade and write effect
         Input()         |          same than Print() but adds an input to the end and returns its valor
-
     """
 
     def Print(text: str, color: list, interval=0.05, hide_cursor: bool = True, end: str = Colors.reset) -> None:
-
         if hide_cursor:
             Cursor.HideCursor()
 
@@ -647,7 +693,6 @@ class Write:
             Cursor.ShowCursor()
 
     def Input(text: str, color: list, interval=0.05, hide_cursor: bool = True, input_color: str = Colors.reset, end: str = Colors.reset) -> str:
-
         if hide_cursor:
             Cursor.HideCursor()
 
@@ -668,10 +713,8 @@ class Write:
     def _write(text: str, color, interval: int):
         lines = list(text)
         if type(color) == list:
-            while True:
-                if len(lines) <= len(color):
-                    break
-                ncolor = [col for col in color]
+            while not len(lines) <= len(color):
+                ncolor = list(color)
                 for col in ncolor:
                     color.append(col)
 
@@ -694,72 +737,96 @@ class Center:
         XCenter()                  |             center the given text in X cords
         YCenter()                  |             center the given text in Y cords
         Center()                   |             center the given text in X and Y cords
+        GroupAlign()               |             align the given text in a group
+        TextAlign()                |             align the given text per lines
 
+    NOTE: the functions of the class can be broken if the text argument has colors in it
     """
 
-    def XCenter(var: str, spaces: int = None, icon: str = " ", middle: bool = False):
+    center = 'CENTER'
+    left = 'LEFT'
+    right = 'RIGHT'
+
+    def XCenter(text: str, spaces: int = None, icon: str = " "):
         if spaces is None:
-            spaces = Center._xspaces(var=var)
+            spaces = Center._xspaces(text=text)
+        return "\n".join((icon * spaces) + text for text in text.splitlines())
 
-        if not middle:
-            return "\n".join((icon * spaces) + var for var in var.splitlines())
-        else:
-            return "\n".join((icon * spaces) + var + (icon * int(spaces)) for var in var.splitlines())
-
-    def YCenter(var: str, spaces: int = None, icon: str = "\n", middle: bool = False):
+    def YCenter(text: str, spaces: int = None, icon: str = "\n"):
         if spaces is None:
-            spaces = Center._yspaces(var=var)
+            spaces = Center._yspaces(text=text)
 
-        if not middle:
-            return icon * spaces + "\n".join(var.splitlines())
-        else:
-            return icon * spaces + "\n".join(var.splitlines()) + icon * spaces
+        return icon * spaces + "\n".join(text.splitlines())
 
-    def Center(var: str, xspaces: int = None, yspaces: int = None, xicon: str = " ", yicon: str = "\n", middle: bool = False) -> str:
+    def Center(text: str, xspaces: int = None, yspaces: int = None, xicon: str = " ", yicon: str = "\n") -> str:
         if xspaces is None:
-            xspaces = Center._xspaces(var=var)
+            xspaces = Center._xspaces(text=text)
 
         if yspaces is None:
-            yspaces = Center._yspaces(var=var)
+            yspaces = Center._yspaces(text=text)
 
-        if not middle:
-            var = yicon * yspaces + "\n".join(var.splitlines())
-        else:
-            var = yicon * yspaces + \
-                "\n".join(var.splitlines()) + yicon * yspaces
+        text = yicon * yspaces + "\n".join(text.splitlines())
+        return "\n".join((xicon * xspaces) + text for text in text.splitlines())
 
-        if not middle:
-            return "\n".join((xicon * xspaces) + var for var in var.splitlines())
+    def GroupAlign(text: str, align: str = center):
+        align = align.upper()
+        if align == Center.center:
+            return Center.XCenter(text)
+        elif align == Center.left:
+            return text
+        elif align == Center.right:
+            length = _terminal_size().columns
+            maxLineSize = max(len(line) for line in text.splitlines())
+            return '\n'.join((' ' * (length - maxLineSize)) + line for line in text.splitlines())
         else:
-            return "\n".join((xicon * xspaces) + var + (xicon * int(xspaces)) for var in var.splitlines())
+            raise Center.BadAlignment()
+    
+    def TextAlign(text: str, align: str = center):
+        align = align.upper()
+        mlen = max(len(i) for i in text.splitlines())
+        if align == Center.center:
+
+            return "\n".join((' ' * int(mlen/2 - len(lin)/2)) + lin for lin in text.splitlines())
+        elif align == Center.left:
+            return text
+        elif align == Center.right:
+            ntext = '\n'.join(' ' * (mlen - len(lin)) + lin for lin in text.splitlines())
+            return ntext
+        else:
+            raise Center.BadAlignment()
+
 
     """ ! developper area ! """
 
-    def _xspaces(var: str):
+    def _xspaces(text: str):
         try:
             col = _terminal_size().columns
         except OSError:
             return 0
-        varl = var.splitlines()
-        nvarl = max(len(v) for v in varl if v.strip())
-        return int((col - nvarl) / 2)
+        textl = text.splitlines()
+        ntextl = max((len(v) for v in textl if v.strip()), default = 0)
+        return int((col - ntextl) / 2)
 
-    def _yspaces(var: str):
+    def _yspaces(text: str):
         try:
             lin = _terminal_size().lines
         except OSError:
             return 0
-        varl = var.splitlines()
-        nvarl = len(varl)
-        return int((lin - nvarl) / 2)
+        textl = text.splitlines()
+        ntextl = len(textl)
+        return int((lin - ntextl) / 2)
 
+    class BadAlignment(Exception):
+        def __init__(self):
+            super().__init__("Choose a correct alignment: Center.center / Center.left / Center.right")
 
 class Add:
+
     """
     1 function:
         Add()           |           allow you to add a text to another, and even center it
-
     """
+
     def Add(banner1, banner2, spaces=0, center=False):
         if center:
             split1 = len(banner1.splitlines())
@@ -772,7 +839,7 @@ class Add:
                 spaces = 0
 
         if spaces > max(len(banner1.splitlines()), len(banner2.splitlines())):
-            # raise Banner.MaximumSpaces(f"Too much spaces [{spaces}].")
+            # raise Banner.MaximumSpaces(spaces)
             spaces = max(len(banner1.splitlines()), len(banner2.splitlines()))
 
         ban1 = banner1.splitlines()
@@ -817,7 +884,8 @@ class Add:
     """ ! developper area ! """
 
     class MaximumSpaces(Exception):
-        ...
+        def __init__(self, spaces: str):
+            super().__init__(f"Too much spaces [{spaces}].")
 
     def _length(ban1):
         bigestline = 0
@@ -831,12 +899,13 @@ class Add:
         return [line + (size - len(line)) * " " for line in ban1]
 
 
-class Box:
+class Banner:
+
     """
     2 functions:
         SimpleCube()                  |             create a simple cube with the given text
         Lines()                       |             create a text framed by two lines
-
+        Arrow()                       |             create a custom arrow
     """
 
     def Box(content: str, up_left: str, up_right: str, down_left: str, down_right: str, left_line: str, up_line: str, right_line: str, down_line: str) -> str:
@@ -874,18 +943,50 @@ class Box:
     def DoubleCube(content: str) -> str:
         return Box.Box(content, "╔═", "═╗", "╚═", "═╝", "║", "═", "║", "═")
 
-    def Lines(content: str) -> str:
-        # voir pour ajouter un argument pour personnaliser la ligne
+    def Lines(content: str, color = None, mode = Colorate.Horizontal, line = '═', pepite = 'ቐ') -> str:
         l = 1
         for c in content.splitlines():
             if len(c) > l:
                 l = len(c)
-        box = f"─{'═'*l}☆☆{'═'*l}─"
-        addspace = round((len(box) - len(content)) / 2)
-        if addspace % 2 == 0:
-            addspace += 1
+        mode = Colorate.Horizontal if color is not None else (lambda **kw: kw['text'])
+        box = mode(text = f"─{line*l}{pepite * 2}{line*l}─", color = color)
+        assembly = box + "\n" + content + "\n" + box
+        final = ''
+        for lines in assembly.splitlines():
+            final += Center.XCenter(lines) + "\n"
+        return final
+    
+    def Arrow(icon: str = 'a', size: int = 2, number: int = 2, direction = 'right') -> str:
+        spaces = ' ' * (size + 1)
+        _arrow = ''
+        structure = (size + 2, [size * 2, size * 2])
+        count = 0
+        if direction == 'right':
+            for i in range(structure[1][0]):
+                line = (structure[0] * icon)
+                _arrow += (' ' * count) + spaces.join([line] * (number)) + '\n'
+                count += 2
 
-        return box + "\n" + addspace * " " + content + addspace * " " + "\n" + box
+            for i in range(structure[1][0] + 1):
+                line = (structure[0] * icon)
+                _arrow += (' ' * count) + spaces.join([line] * (number)) + '\n'
+                count -= 2
+        elif direction == 'left':
+            for i in range(structure[1][0]):
+                count += 2
 
+            for i in range(structure[1][0]):
+                line = (structure[0] * icon)
+                _arrow += (' ' * count) + spaces.join([line] * (number)) + '\n'
+                count -= 2
+
+            for i in range(structure[1][0] + 1):
+                line = (structure[0] * icon)
+                _arrow += (' ' * count) + spaces.join([line] * (number)) + '\n'
+                count += 2
+        return _arrow
+
+
+Box = Banner
 
 System.Init()
\ No newline at end of file
